==> splitpipe-0.3.tar.gz: splitpipe-0.3/display.cc <==
/*
    Copyright (C) 2005  Netherlabs Computer Consulting BV

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as published by
    the Free Software Foundation;

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "display.hh"
#include <ncurses.h>
#include <string>
#include <stdexcept>
#include <time.h>

using namespace std;

SplitpipeDisplay::SplitpipeDisplay()
{
  char *term=getenv("TERM");
  if(term && !strcmp(term, "xterm")) 
    putenv("TERM=vt102");  // make sure ncurses does not clear screen on exit

  /* (See above) 
22:09 <zwane> hehehe
22:09 <zwane> *yuck*
22:10 <zwane> good god
22:10 <ahu> there are rumours there is a beter solution
22:10 <zwane> you should be flogged for that offence
22:10 <ahu> at least I admit my weaknesses
22:10 <zwane> ahu: if i did, i'd be flogged publically daily
22:10 <sarnold> ahu: eww :)
22:10 <zwane> so i keep my hacks _under_wraps_ ;)
  */

  initscr();      /* initialize the curses library */

  int logSize=10;
  int statusSize=5;
  
  d_pwin=newwin(getHeight()-logSize-statusSize, 0, 0, 0);
  d_sepawin=newwin(statusSize, 0, getHeight()-statusSize-logSize, 0);
  d_logwin=newwin(logSize, 0, getHeight()-logSize, 0);
  
  scrollok(d_logwin, true);
  scrollok(d_pwin, true);
  if(!d_pwin || !d_logwin) {
    throw runtime_error("Unable to initialize ncurses");
  }
  
  //  keypad(swin, TRUE);  /* enable keyboard mapping */
  nonl();         /* tell curses not to do NL->CR/NL on output */
  //cbreak();       /* take input chars one at a time, no wait for \n */
  noecho();
  
  wmove(d_sepawin,0,0);
  
  for(int x=0;x<getWidth();++x)
    waddch(d_sepawin,ACS_HLINE);
  
  wmove(d_sepawin,2,0);
  for(int x=0;x<getWidth();++x)
    waddch(d_sepawin,ACS_HLINE);
  wmove(d_sepawin,4,0);
  for(int x=0;x<getWidth();++x)
    waddch(d_sepawin,ACS_HLINE);
  
  //  mvwprintw(d_sepawin,3,0,"Volume done ETA: --:--");
  
  wrefresh(d_sepawin);
  wrefresh(d_logwin);
  wrefresh(d_pwin);

  d_firstlog=true;
}

void SplitpipeDisplay::setTotalBytes(uint64_t input, uint64_t output, uint32_t bufferSize, int volPerc)
{
  mvwprintw(d_sepawin,3,0,"Input: %5dMB     Output: %5dMB     Buffered: %5dKB     Volume done: %3d%%", 
	    (int)(input/1000000), (int)(output/1000000), bufferSize/1024, volPerc);
  wrefresh(d_sepawin);
  wrefresh(d_logwin);
}

void SplitpipeDisplay::setBarPercentage(int percentage)
{
  int width=getWidth()-14;
  int fillTo=width*percentage/100.0;
  mvwprintw(d_sepawin,1,0,"Buffer: %3d%%", percentage);

  wmove(d_sepawin,1,14);
  for(int x=0 ; x < width;++x)
    if(x<fillTo)
      waddch(d_sepawin,'X');
    else
      waddch(d_sepawin,' ');
  wrefresh(d_sepawin);
  wrefresh(d_logwin);
}

SplitpipeDisplay::~SplitpipeDisplay()
{
  endwin();
}

void SplitpipeDisplay::refresh()
{
  redrawwin(d_pwin);
  redrawwin(d_sepawin);
  redrawwin(d_logwin);
  wrefresh(curscr);
}
  
void SplitpipeDisplay::programPut(char c)
{
  waddch(d_pwin, c);

}

void SplitpipeDisplay::programCommit()
{
  wrefresh(d_pwin);
  wrefresh(d_logwin);
}

void SplitpipeDisplay::setLogStandout(bool bold)
{
  if(bold)
    wattron(d_logwin, A_BOLD);
  else
    wattroff(d_logwin, A_BOLD);
}

void SplitpipeDisplay::log(const char* fmt, ...)
{
  char buffer[40];
  struct tm tm;
  time_t t;
  time(&t);
  tm=*localtime(&t);

  strftime(buffer,sizeof(buffer),"%H:%M:%S ", &tm);

  string total;
  if(!d_firstlog) {
    total="\n";
  }
  d_firstlog=false;
  total+=buffer;
  total+=fmt;

  va_list ap;
    
  /* Try to print in the allocated space. */
  va_start(ap, fmt);
  vw_printw(d_logwin, (char*)total.c_str(), ap); // cast is needed for Solaris
  va_end(ap);
  wrefresh(d_logwin);
}

int SplitpipeDisplay::getHeight()
{
  int y,x;
  getmaxyx(stdscr, y,x);
  return y;
}
  

int SplitpipeDisplay::getWidth()
{
  int y,x;
  getmaxyx(stdscr, y,x);
  return x;
}
/*
int main(int argc, char** argv)
{
  SplitpipeDisplay spd;
  spd.log("Gaat dit allemaal '%s'\n", "goed");
  spd.programPut('h');
  spd.setBarPercentage(88);
  sleep(2);
}
*/

==> splitpipe-0.4.tar.gz: splitpipe-0.4/display.cc <==
/*
    Copyright (C) 2005  Netherlabs Computer Consulting BV

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as published by
    the Free Software Foundation;

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "display.hh"
#include <ncurses.h>
#include <string>
#include <stdexcept>
#include <time.h>

using namespace std;

SplitpipeDisplay::SplitpipeDisplay() : d_inSpeed(100), d_outSpeed(100)
{
  char *term=getenv("TERM");
  if(term && !strcmp(term, "xterm")) 
    putenv("TERM=vt102");  // make sure ncurses does not clear screen on exit

  /* (See above) 
22:09 <zwane> hehehe
22:09 <zwane> *yuck*
22:10 <zwane> good god
22:10 <ahu> there are rumours there is a beter solution
22:10 <zwane> you should be flogged for that offence
22:10 <ahu> at least I admit my weaknesses
22:10 <zwane> ahu: if i did, i'd be flogged publically daily
22:10 <sarnold> ahu: eww :)
22:10 <zwane> so i keep my hacks _under_wraps_ ;)
  */

  initscr();      /* initialize the curses library */

  int logSize=10;
  int statusSize=6;
  
  d_pwin=newwin(getHeight()-logSize-statusSize, 0, 0, 0);
  d_sepawin=newwin(statusSize, 0, getHeight()-statusSize-logSize, 0);
  d_logwin=newwin(logSize, 0, getHeight()-logSize, 0);
  
  scrollok(d_logwin, true);
  scrollok(d_pwin, true);
  if(!d_pwin || !d_logwin) {
    throw runtime_error("Unable to initialize ncurses");
  }
  
  //  keypad(swin, TRUE);  /* enable keyboard mapping */
  nonl();         /* tell curses not to do NL->CR/NL on output */
  //cbreak();       /* take input chars one at a time, no wait for \n */
  noecho();
  
  wmove(d_sepawin,0,0);
  
  for(int x=0;x<getWidth();++x)
    waddch(d_sepawin,ACS_HLINE);
  
  wmove(d_sepawin,2,0);
  for(int x=0;x<getWidth();++x)
    waddch(d_sepawin,ACS_HLINE);
  wmove(d_sepawin,5,0);
  for(int x=0;x<getWidth();++x)
    waddch(d_sepawin,ACS_HLINE);
  
  //  mvwprintw(d_sepawin,3,0,"Volume done ETA: --:--");
  
  wrefresh(d_sepawin);
  wrefresh(d_logwin);
  wrefresh(d_pwin);

  d_firstlog=true;
}

string SplitpipeDisplay::timeLeft(uint64_t bytes,uint32_t bps)
{
  char buffer[80];
  if(!bps) {
    return "--";
  }

  uint64_t seconds=bytes/bps;
  int iseconds=seconds;

  if(seconds < 60)
    snprintf(buffer, sizeof(buffer)-1, "%d seconds", iseconds);
  else if(seconds < 3600)
    snprintf(buffer, sizeof(buffer)-1, "%d minutes", iseconds/60);
  else if(seconds < 86400)
    snprintf(buffer, sizeof(buffer)-1, "%dh, %dm", iseconds/3600, (iseconds % 3600)/60);
  else
    snprintf(buffer, sizeof(buffer)-1, "> 1 day");
  
  return buffer;
}

void SplitpipeDisplay::setTotalBytes(uint64_t input, uint64_t output, uint32_t bufferSize, int volPerc, uint64_t bytesLeft)
{
  mvwprintw(d_sepawin,3,0,"Input: %5dMB     Output: %5dMB     Buffered: %5dKB     Volume done: %3d%%", 
	    (int)(input/1000000), (int)(output/1000000), 
	    bufferSize/1024, volPerc);

  mvwprintw(d_sepawin,4,0,"       %5dKB/s           %5dKB/s             %5dKB/s   Vol ETA: %-10s", 
    	    (unsigned int)(d_inSpeed.getAverage(30)/1000),  (unsigned int)(d_outSpeed.getAverage(30)/1000),
    	    (unsigned int)((d_inSpeed.getAverage(30)/1000.0) - (d_outSpeed.getAverage(30)/1000.0)),
	    timeLeft(bytesLeft, (uint32_t)d_outSpeed.getAverage(60)).c_str());

  
  d_inSpeed.feed(input);
  d_outSpeed.feed(output);

  wrefresh(d_sepawin);
  wrefresh(d_logwin);
}

void SplitpipeDisplay::setBarPercentage(int percentage)
{
  int width=getWidth()-14;
  int fillTo=width*percentage/100.0;
  mvwprintw(d_sepawin,1,0,"Buffer: %3d%%", percentage);

  wmove(d_sepawin,1,14);
  for(int x=0 ; x < width;++x)
    if(x<fillTo)
      waddch(d_sepawin,'X');
    else
      waddch(d_sepawin,' ');
  wrefresh(d_sepawin);
  wrefresh(d_logwin);
}

SplitpipeDisplay::~SplitpipeDisplay()
{
  endwin();
}

void SplitpipeDisplay::refresh()
{
  redrawwin(d_pwin);
  redrawwin(d_sepawin);
  redrawwin(d_logwin);
  wrefresh(curscr);
}
  
void SplitpipeDisplay::programPut(char c)
{
  waddch(d_pwin, c);

}

void SplitpipeDisplay::programCommit()
{
  wrefresh(d_pwin);
  wrefresh(d_logwin);
}

void SplitpipeDisplay::setLogStandout(bool bold)
{
  if(bold)
    wattron(d_logwin, A_BOLD);
  else
    wattroff(d_logwin, A_BOLD);
}

void SplitpipeDisplay::log(const char* fmt, ...)
{
  char buffer[40];
  struct tm tm;
  time_t t;
  time(&t);
  tm=*localtime(&t);

  strftime(buffer,sizeof(buffer),"%H:%M:%S ", &tm);

  string total;
  if(!d_firstlog) {
    total="\n";
  }
  d_firstlog=false;
  total+=buffer;
  total+=fmt;

  va_list ap;
    
  /* Try to print in the allocated space. */
  va_start(ap, fmt);
  vw_printw(d_logwin, (char*)total.c_str(), ap); // cast is needed for Solaris
  va_end(ap);
  wrefresh(d_logwin);
}

int SplitpipeDisplay::getHeight()
{
  int y,x;
  getmaxyx(stdscr, y,x);
  return y;
}
  

int SplitpipeDisplay::getWidth()
{
  int y,x;
  getmaxyx(stdscr, y,x);
  return x;
}
/*
int main(int argc, char** argv)
{
  SplitpipeDisplay spd;
  spd.log("Gaat dit allemaal '%s'\n", "goed");
  spd.programPut('h');
  spd.setBarPercentage(88);
  sleep(2);
}
*/
